%DRAWLINE   Draws a line in an image
%
% SYNOPSIS:
%  image_out = drawline(image_in, start, end, intensity)
%
%  start:     array containing the start position(s) of the line(s)
%  end:       array containing the end position(s) of the line(s)
%  intensity: intensity of the line(s), either one or one per line
%
% DEFAULTS:
%  intensity = 255;
%
% EXAMPLE:
%  drawline(newim,[100 100; 0 200],[200 200; 200 0],[1 2])

% (C) Copyright 1999-2001               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Bernd Rieger, Apr 2001
% July 2007, extended to use for multiple lines (BR)

function out = drawline(varargin)

d = struct('menu','Generation',...
           'display','Add line',...
           'inparams',struct('name',       {'in',         'st',    'ed',      'inten'},...
                             'description',{'Input image','Start Coordinates','End Coordinates','Intensity'},...
                             'type',       {'image',      'image',    'image',     'array',},...
                             'dim_check',  {0,            0,           0,           -1, },...
                             'range_check',{[],           'R+',        'R+',         'R'},...
                             'required',   {1,            1,           1,           0},...
                             'default',    {'a',          '[0 0]',  '[255 255]',           255}...
                            ),...
           'outparams',struct('name',{'out'},...
                              'description',{'Output image'},...
                              'type',{'image'}...
                              )...
         );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [in,st,ed,inten] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

st = double(st);
ed = double(ed);

if strcmp(class(in),'dip_image_array')
   out=in;
   if length(inten)==1
      for ii=1:length(in)
         out{ii}=dip_drawline(in{ii},st, ed, inten);
      end
   else
      if length(inten)~=length(in)
         error(['Number of elements in intensity vector ',...
                'must match the number of colour channels']);
      end
      for ii=1:length(in)
         out{ii}=dip_drawline(in{ii},st, ed, inten(ii));
      end
   end
   colorspace(out,colorspace(in));
else
   if any(size(st)~=size(ed))
      error('Must have the same number of start and end points.');
   end
   if min(size(st))==1
   %one line
      if length(inten)~=1 
         error('For grey-value images only scalar intensity values are supported');
      end
      out = dip_drawline(in, st, ed, inten);
   else
   %several lines
      coI = newim([ndims(in) 2 size(st,1)]);
      for ii=1:size(st,1)
         coI(:,0,ii-1) = st(ii,:);
         coI(:,1,ii-1) = ed(ii,:);
      end
      out = dip_drawlines(in,coI,inten);%undocumented MvG private code
   end
end
